@@ -5,21 +5,17 @@
 # - Each helper gets a reference to this command object.
 
 
-import sys, os, platform, re, subprocess, logging
+import sys, os, platform, re, subprocess, logging, shlex
 from datetime import datetime
 from pathlib import Path
+from importlib import import_module
 
 from django.core.management.base import BaseCommand, CommandError
 from django.conf import settings
 
-from simple_deploy.management.commands.utils import deploy_messages as d_msgs
-from simple_deploy.management.commands.utils import deploy_messages_heroku as dh_msgs
-from simple_deploy.management.commands.utils import deploy_messages_platformsh as plsh_msgs
-from simple_deploy.management.commands.utils import deploy_messages_flyio as flyio_msgs
-
-from simple_deploy.management.commands.utils.deploy_heroku import HerokuDeployer
-from simple_deploy.management.commands.utils.deploy_platformsh import PlatformshDeployer
-from simple_deploy.management.commands.utils.deploy_flyio import FlyioDeployer
+from . import deploy_messages as d_msgs
+from .utils import write_file_from_template
+from . import cli
 
 
 class Command(BaseCommand):
@@ -27,55 +23,35 @@ class Command(BaseCommand):
     Configure as much as possible automatically.
     """
 
+    # Provide a summary of simple_deploy in the help text.
+    help = "Configures your project for deployment to the specified platform."
+
+    def __init__(self):
+        """Customize help output."""
+
+        # Keep default BaseCommand args out of our help text.
+        self.suppressed_base_arguments.update([
+            '--version', '-v', '--settings', '--pythonpath', '--traceback', '--no-color',
+            '--force-color'
+            ])
+        # Ensure that --skip-checks is not included in help output.
+        self.requires_system_checks = []
+
+        super().__init__()
+
+
+    def create_parser(self, prog_name, subcommand, **kwargs):
+        """Customize the ArgumentParser object that will be created."""
+        epilog = "For more help, see the full documentation at: https://django-simple-deploy.readthedocs.io"
+        parser = super().create_parser(prog_name, subcommand, usage=cli.get_usage(),
+                epilog=epilog, add_help=False, **kwargs)
+
+        return parser
+
+
     def add_arguments(self, parser):
         """Define CLI options."""
-
-        # --- Platform-agnostic arguments ---
-
-        parser.add_argument('--automate-all',
-            help="Automate all aspects of deployment?",
-            action='store_true')
-
-        parser.add_argument('--platform', type=str,
-            help="Which platform do you want to deploy to?",
-            default='')
-
-        # Allow users to skip logging.
-        parser.add_argument('--no-logging',
-            help="Do you want a record of simple_deploy's output?",
-            action='store_true')
-
-        # Allow users to use simple_deploy even with an unclean git status.
-        parser.add_argument('--ignore-unclean-git',
-            help="Run simple_deploy even with an unclean `git status` message.",
-            action='store_true')
-
-        # --- Platform.sh arguments ---
-
-        # Allow users to set the deployed project name. This is the name that
-        #   will be used by the platform, which may be different than the name
-        #   used in the `startproject` command. See the Platform.sh script
-        #   for use of this flag.
-        parser.add_argument('--deployed-project-name', type=str,
-            help="What name should the platform use for this project?\n(This is normally discovered automatically through inspection.)",
-            default='')
-
-        # Allow users to specify the region for a project when using --automate-all.
-        parser.add_argument('--region', type=str,
-            help="Which region do you want to deploy to?",
-            default='us-3.platform.sh')
-
-        # --- Developer arguments ---
-
-        # If we're doing local unit testing, we need to avoid some network
-        #   calls.
-        parser.add_argument('--unit-testing',
-            help="Used for local unit testing, to avoid network calls.",
-            action='store_true')
-
-        parser.add_argument('--integration-testing',
-            help="Used for integration testing, to avoid confirmations.",
-            action='store_true')
+        sd_cli = cli.SimpleDeployCLI(parser)
 
 
     def handle(self, *args, **options):
@@ -83,6 +59,7 @@ class Command(BaseCommand):
         self.stdout.write("Configuring project for deployment...")
 
         # Parse CLI options, and validate the set of arguments we've been given.
+        #   _validate_command() instantiates a PlatformDeployer object.
         self._parse_cli_options(options)
         self._validate_command()
 
@@ -95,12 +72,11 @@ class Command(BaseCommand):
         self._inspect_project()
 
         # Confirm --automate-all, if needed. Currently, this needs to happen before
-        #   _validate_platform(), because fly_io takes action based on automate_all
+        #   validate_platform(), because fly_io takes action based on automate_all
         #   in _validate_platform().
-        # Then build the platform-specifc deployer instance, and do platform-specific
-        #   validation. 
+        # Then do platform-specific validation.
         self._confirm_automate_all()
-        self._validate_platform()
+        self.platform_deployer.validate_platform()
 
         # All validation has been completed. Make platform-agnostic modifications.
         # Start with logging.
@@ -118,10 +94,6 @@ class Command(BaseCommand):
 
         self._add_simple_deploy_req()
 
-        # During development, sometimes helpful to exit before calling deploy().
-        # print('bye')
-        # sys.exit()
-
         # All platform-agnostic work has been completed. Call platform-specific
         #   deploy() method.
         self.platform_deployer.deploy()
@@ -135,6 +107,7 @@ class Command(BaseCommand):
         # Platform-agnostic arguments.
         self.automate_all = options['automate_all']
         self.platform = options['platform']
+        
         # This is a True-to-disable option; turn it into a more intuitive flag?
         self.log_output = not(options['no_logging'])
         self.ignore_unclean_git = options['ignore_unclean_git']
@@ -150,33 +123,38 @@ class Command(BaseCommand):
 
     def _validate_command(self):
         """Make sure the command has been called with a valid set of arguments.
+        Returns:
+        - None
+        - Calls methods that will raise a CommandError if the command is invalid.
+        """
+        # Right now, we're just validating the platform argument. There will be
+        #   more validation, so keep this method in place.
+        self._validate_platform_arg()
+
+
+    def _validate_platform_arg(self):
+        """Find out which platform we're targeting, instantiate the
+        platform-specific deployer object and platform-specific messages, and 
+        get confirmation about using a platform with preliminary support.
         """
         if not self.platform:
-            self.write_output(d_msgs.requires_platform_flag, write_to_console=False)
             raise CommandError(d_msgs.requires_platform_flag)
-
-
-    def _validate_platform(self):
-        """Find out which platform we're targeting, and instantiate the
-        platform-specific deployer object. Also, call any necessary
-        platform-specific validation and confirmation methods here.
-        """
-        if self.platform == 'heroku':
-            self.write_output("  Targeting Heroku deployment...", skip_logging=True)
-            self.platform_deployer = HerokuDeployer(self)
-        elif self.platform == 'platform_sh':
-            self.write_output("  Targeting platform.sh deployment...", skip_logging=True)
-            self.platform_deployer = PlatformshDeployer(self)
-            self.platform_deployer.confirm_preliminary()
-        elif self.platform == 'fly_io':
-            self.write_output("  Targeting Fly.io deployment...", skip_logging=True)
-            self.platform_deployer = FlyioDeployer(self)
-            self.platform_deployer.confirm_preliminary()
+        elif self.platform in ['fly_io', 'platform_sh', 'heroku']:
+            self.write_output(f"  Deployment target: {self.platform}", skip_logging=True)
         else:
-            error_msg = f"The platform {self.platform} is not currently supported."
+            error_msg = d_msgs.invalid_platform_msg(self.platform)
             raise CommandError(error_msg)
 
-        self.platform_deployer.validate_platform()
+        self.platform_msgs = import_module(f".{self.platform}.deploy_messages", package='simple_deploy.management.commands')
+
+        deployer_module = import_module(f".{self.platform}.deploy", package='simple_deploy.management.commands')
+        self.platform_deployer = deployer_module.PlatformDeployer(self)
+
+        try:
+            self.platform_deployer.confirm_preliminary()
+        except AttributeError:
+            # The targeted platform does not have a preliminary warning.
+            pass
 
 
     def _confirm_automate_all(self):
@@ -188,23 +166,8 @@ class Command(BaseCommand):
         if not self.automate_all:
             return
 
-        # Confirm the user knows exactly what will be automated; this
-        #   message is specific to each platform.
-        if self.platform == 'heroku':
-            msg = dh_msgs.confirm_automate_all
-        elif self.platform == 'platform_sh':
-            msg = plsh_msgs.confirm_automate_all
-        elif self.platform == 'fly_io':
-            msg = flyio_msgs.confirm_automate_all
-        else:
-            # The platform name is not valid!
-            # DEV: This should be removed when the logic around when to call
-            #   _validate_platform() has been cleaned up.
-            # See issue #120: https://github.com/ehmatthes/django-simple-deploy/issues/120
-            error_msg = f"The platform {self.platform} is not currently supported."
-            raise CommandError(error_msg)
-
-        self.write_output(msg, skip_logging=True)
+        # Confirm the user knows exactly what will be automated.
+        self.write_output(self.platform_msgs.confirm_automate_all, skip_logging=True)
         confirmed = self.get_confirmation(skip_logging=True)
 
         if confirmed:
@@ -619,26 +582,6 @@ class Command(BaseCommand):
                 logging.info(line)
 
 
-    def execute_subp_run_parts(self, cmd_parts):
-        """This is similar to execute_subp_run(), but it receives a list of
-        command parts rather than a string command. Having this separate method
-        is cleaner than having nested if statements in execute_subp_run().
-
-        Currently this is used to issue a git commit command, where running
-          cmd.split() would split on the commit message.
-
-        DEV: May want to make execute_subp_run() examine cmd that's received,
-        and dispatch the work based on whether it receives a string or sequence.
-        """
-        if self.on_windows:
-            cmd_string = ' '.join(cmd_parts)
-            output = subprocess.run(cmd_string, shell=True, capture_output=True)
-        else:
-            output = subprocess.run(cmd_parts, capture_output=True)
-
-        return output
-
-
     def execute_subp_run(self, cmd, check=False):
         """Execute subprocess.run() command.
         We're running commands differently on Windows, so this method
@@ -651,12 +594,13 @@ class Command(BaseCommand):
 
         Returns:
             - CompletedProcess instance
-            - if check=True is passed, raises CalledProcessError. 
+            - if check=True is passed, raises CalledProcessError instead of
+              CompletedProcess with an error code.
         """
         if self.on_windows:
             output = subprocess.run(cmd, shell=True, capture_output=True)
         else:
-            cmd_parts = cmd.split()
+            cmd_parts = shlex.split(cmd)
             output = subprocess.run(cmd_parts, capture_output=True, check=check)
 
         return output
@@ -675,8 +619,9 @@ class Command(BaseCommand):
         # DEV: This only captures stderr right now.
         #   This is used for commands that run long enough that we don't
         #   want to use a simple subprocess.run(capture_output=True). Right
-        #   now that's only the `git push heroku` call. That call writes to
-        #   stderr; I'm not sure how to stream both stdout and stderr.
+        #   now that's the `git push heroku` call. That call writes to
+        #   stderr; I'm not sure how to stream both stdout and stderr. It also
+        #   affects `platform create` and `platform push`.
         #
         #     This will also be needed for long-running steps on other platforms,
         #   which may or may not write to stderr. Adding a parameter
@@ -758,11 +703,11 @@ class Command(BaseCommand):
             return
 
         self.write_output("  Committing changes...")
+
         cmd = 'git add .'
         output = self.execute_subp_run(cmd)
         self.write_output(output)
-        # If we write this command as a string, the commit message will be split
-        #   incorrectly.
-        cmd_parts = ['git', 'commit', '-am', '"Configured project for deployment."']
-        output = self.execute_subp_run_parts(cmd_parts)
-        self.write_output(output)
+
+        cmd = 'git commit -am "Configured project for deployment."'
+        output = self.execute_subp_run(cmd)
+        self.write_output(output)
\ No newline at end of file
